from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, User
from typing import Dict, List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ApplySignatureInput(TcBaseObj):
    """
    Structure represents the parameters required to apply digital signature on target objects.
    
    :var object: The target object whose digital signature will be applied.
    :var base64String: The Base64 string containing signed information.
    """
    object: BusinessObject = None
    base64String: str = ''


@dataclass
class PerformActionInputInfo(TcBaseObj):
    """
    Structure represents the parameters required to perform an action on an EPMTask or Signoff.
    
    :var clientId: Input string to uniquely identify the input, used primarily for error handling and output mapping.
    
    :var actionableObject: The EPMTask or Signoff on which the action needs to be performed.
    :var action: Action to be performed on EPMTask or Signoff.
    
    Possible values for action are as follows.
    
                    Action                        Description
    
    "SOA_EPM_assign_action"- Assign task or delegate a signoff
    "SOA_EPM_start_action"- Start task
    "SOA_EPM_complete_action"- Mark the task as Complete
    "SOA_EPM_skip_action"- Skip task
    "SOA_EPM_suspend_action"- Suspend task
    "SOA_EPM_resume_action"    - Resume task
    "SOA_EPM_undo_action"- Undo task
    "SOA_EPM_abort_action"- Abort task
    "SOA_EPM_perform_action"- Perform task
    "SOA_EPM_approve_action"- Approve a Review Task
    "SOA_EPM_reject_action"- Reject a Review Task
    "SOA_EPM_promote_action"- Promote task
    "SOA_EPM_demote_action"- Demote task
    "SOA_EPM_assign_approver_action"- This action is not used. Use addSignoffs operations to add users to
    select-signoff-team task
    "SOA_EPM_notify_action"- Notify
    "SOA_EPM_no_action"- No action, results in the comments being set without triggering state change on the task.
    "SOA_EPM_fail_action"- Fail the task
    "SOA_EPM_claim_action"- Claim the task
    :var password: The password for a secure Task. The value of this parameter is ignored for non-secure Task.
    
    :var supportingValue: This argument can be used to send in decision value or result.  If the task is
    perform-signoff task, possible values for decision are:
    - "SOA_EPM_no_decision"
    - "SOA_EPM_approve"
    - "SOA_EPM_reject"
    
    
    
    Following are the result values applicable for different tasks:
    
    - "SOA_EPM_unset"- Do Task, Review Task, Route Task, Ackowledge Task, EPMTask, Condition Task (Auto/Manual),
    select-signoff-team task, Validate Task
    - "SOA_EPM_completed"- Do Task, EPMTask, select-signoff-team task
    - "SOA_EPM_unable_to_complete"- Do Task, EPMTtask, Condition Task (Manual)
    - "SOA_EPM_true"- Condition Task (Auto/Manual)
    - "SOA_EPM_false"- Condition Task (Auto/Manual)
    - "SOA_EPM_no_error"- Validate Task
    
    
    :var supportingObject: If the action is assign, provide a User or ResourcePool to assign the task.
    :var propertyNameValues: Property name and values map that will contain all property names and corresponding string
    values that needs to be saved. e.g. comments can be set on the EPMTask/Signoff by adding it to this map.
    :var signatures: List of ApplySignatureInput objects, each representing target object and its corresponding Base64
    string.
    """
    clientId: str = ''
    actionableObject: BusinessObject = None
    action: str = ''
    password: str = ''
    supportingValue: str = ''
    supportingObject: BusinessObject = None
    propertyNameValues: NameValuePair = None
    signatures: List[ApplySignatureInput] = ()


@dataclass
class SetActiveSurrogateInputInfo(TcBaseObj):
    """
    Structure represents the parameters required to set/unset the logged in user as an active surrogate on the EPMTask
    or Signoff objects and transfer the checkout of the target objects to/from the original user.
    
    :var taskOrSignoffTag: The EPMTask or Signoff business object for which the current user needs to be set/unset as
    an active surrogate.
    :var releaseStandIn: Logged in user is set as an active surrogate on EPMTask or Signoff when value of this
    parameter is false and is released as an active surrogate from the EPMTask or Signoff when value of this parameter
    is true.
    :var transferCheckouts: During Stand-In operation, if the value of this parameter is true, the transfer of checkout
    of the target object(s) from the original user to the current user is performed. During Release operation, if the
    value of this  parameter is true, the transfer of checkout of the target object(s) from the current user to the
    original user is performed. If the value of the parameter is false, transfer of checkout of the target object(s) is
    not performed during Stand-In or Release operation.
    """
    taskOrSignoffTag: BusinessObject = None
    releaseStandIn: bool = False
    transferCheckouts: bool = False


@dataclass
class SurrogateInput(TcBaseObj):
    """
    This operation sets or unsets the surrogate resource for a given User effective for a given date range.
    
    :var unset: If true, remove surrogate; if false, set surrogate.
    :var fromResource: The user for whom the surrogate will be set/unset.
    :var toResource: The surrogate user.
    :var startDate: The date at which the surrogate setting is to take effect.
    :var endDate: The date at which the surrogate setting ceases to be in effect.
    """
    unset: bool = False
    fromResource: User = None
    toResource: User = None
    startDate: datetime = None
    endDate: datetime = None


"""
This map holds property name and vector of property values. This is a generic container that contains the property name as string and the value is the string representation of the property value.
"""
NameValuePair = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from typing import Dict, List


@dataclass
class GetClassificationHierarchiesResponse(TcBaseObj):
    """
    Holds the classification hierarchies returned by the  'getClassificationHierarchies()' method.
    
    :var hierarchies: References a map of class names associated with the given WorkspaceObject.
    :var ids: References a map of class IDs found associated with the given WorkspaceObject.
    :var data: Any failures with WorkspaceObject ID mapped to the error message are returned in the ServiceData list of
    partial errors.
    """
    hierarchies: ClassificationHierarchiesMap = None
    ids: ClassificationHierarchiesMap = None
    data: ServiceData = None


@dataclass
class GetClassificationPropertiesResponse(TcBaseObj):
    """
    Holds the classification properties returned by the  'getClassificationProperties()' method.
    
    :var propnames: References a map of classification property names for this WorkspaceObject object, if it is
    classified.
    :var propvalues: References a map of classification property values for this WorkspaceObject object, if it is
    classified.
    :var hierarchies: References a map of class IDs found for this WorkspaceObject object, if it is classified.
    :var propDeprecatedflags: References a map of of deprecated flags for the associated classification property. This
    flag will always be FALSE in case of non-key-LOV type of classification properties
    :var data: Any failures with the WorkspaceObject ID mapped to the error message are returned in the ServiceData
    list of partial errors.
    """
    propnames: ClassificationPropertiesMap = None
    propvalues: ClassificationPropertiesMap = None
    hierarchies: ClassificationPropertiesMap = None
    propDeprecatedflags: DeprecatedFlagsMap = None
    data: ServiceData = None


"""
Structure elements:

- primary key - Alphanumeric attribute ID of the classification property.
- values - Deprecated flags for this classification property.


"""
DeprecatedFlagsMap = Dict[str, List[bool]]


"""
Map that stores the classification hierarchy information.
"""
ClassificationHierarchiesMap = Dict[str, List[str]]


"""
Structure elements:

- primary key - Alphanumeric attribute ID of the classification property.
- values - Values for this classification property.


"""
ClassificationPropertiesMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Internal.ProductConfiguratorAw._2020_12.ConfiguratorManagement import SetVariantExpressionDataInput, ValidateProductConfigsInput, VariantConfigurationViewResponse2, VariantConfigurationViewIn2, VariantExpressionDataInput2, VariantExpressionDataResponse2, ValidateProductConfigsResponse2
from tcsoa.base import TcService


class ConfiguratorManagementService(TcService):

    @classmethod
    def variantConfigurationView2(cls, input: VariantConfigurationViewIn2) -> VariantConfigurationViewResponse2:
        """
        This operation retrieves variant configuration data available in the current Configurator Context
        required to create a custom variant rule.
        For a given Cfg0ConfiguratorPerspective and selected context or a seed variant rule, the following is retrieved:
        &bull;The valid selections for given groups by optionally applying constraints
        &bull;Violations and other indicators like incomplete, invalid, package.
        &bull;The payload that should be retained by the client for subsequent calls.
        """
        return cls.execute_soa_method(
            method_name='variantConfigurationView2',
            library='Internal-ProductConfiguratorAw',
            service_date='2020_12',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=VariantConfigurationViewResponse2,
        )

    @classmethod
    def getVariantExpressionData2(cls, variantExpressionDataInput: VariantExpressionDataInput2) -> VariantExpressionDataResponse2:
        """
        This operation returns the variant configuration expression data of the input objects.
        """
        return cls.execute_soa_method(
            method_name='getVariantExpressionData2',
            library='Internal-ProductConfiguratorAw',
            service_date='2020_12',
            service_name='ConfiguratorManagement',
            params={'variantExpressionDataInput': variantExpressionDataInput},
            response_cls=VariantExpressionDataResponse2,
        )

    @classmethod
    def setVariantExpressionData2(cls, input: SetVariantExpressionDataInput) -> ServiceData:
        """
        This operation sets the configuration expressions for the input business objects. The variant expressions can
        be set for variant conditions, rules and variant criteria.
        """
        return cls.execute_soa_method(
            method_name='setVariantExpressionData2',
            library='Internal-ProductConfiguratorAw',
            service_date='2020_12',
            service_name='ConfiguratorManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def validateProductConfigurations2(cls, validateProductConfigsInput: ValidateProductConfigsInput) -> ValidateProductConfigsResponse2:
        """
        This operation validates the input selections by applying the constraint rules such as inclusion rule, 
        exclusion rule or feasibility rule for the input configurator perspective.
        
        The operation also returns validation status, violations and selections by considering input selections and the
        rules in the context. The response parameter criteriaStatus conveys the information about the status of the
        configuration which is either valid or invalid. The parameter valueToViolationsMap represents map of values
        having voilations. Each value in this map have list of vloilations.
        
        Exceptions:
        >The following error code may be returned:
        77073    The operation has failed, because an invalid Configurator Perspective was passed.
        77074    The operation must contain a product item in the Configurator Perspective.
        256054 The value for the applyConstraints bit pattern is invalid.
        77146    The operation has failed, because invalid key/value is passed in requestInfo.
        """
        return cls.execute_soa_method(
            method_name='validateProductConfigurations2',
            library='Internal-ProductConfiguratorAw',
            service_date='2020_12',
            service_name='ConfiguratorManagement',
            params={'validateProductConfigsInput': validateProductConfigsInput},
            response_cls=ValidateProductConfigsResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetSharedCommonClientFilesResponse(TcBaseObj):
    """
    The response structure for the getSharedCommonClientFiles operation.  This response contains a map of dataset name
    to DatasetInfo structures.  It also contains the serviceData.
    
    :var datasetInfoMap: This field contains the map of dataset names to thier DatasetInfo objects.
    :var serviceData: The SOA service data, this structure contains any partial errors encountered during processing.
    """
    datasetInfoMap: DatasetInfoMap = None
    serviceData: ServiceData = None


@dataclass
class GetStylesheetPref(TcBaseObj):
    """
    This structure defines the set of preferences that the client maybe defined for the operation.
    
    :var returnThumbnailTickets: If true the thumbnail FMS file tickets are returned.
    :var stylesheetFormat: The stylesheet format to be returned. Legal values are "XRTOnly", "HTMLOnly", or
    "HTMLPreferred".
    
    XRTOnly - Only XRT format is returned, no attempt to return HTML is made.
    
    HTMLOnly - Only the HTML format is returned, even in error situations.
    
    HTMLPreferred - HTML format is returned, except in error situations or when unsupported element or rendering hints
    are found in XRT.
    """
    returnThumbnailTickets: bool = False
    stylesheetFormat: str = ''


@dataclass
class GetStylesheetResponse(TcBaseObj):
    """
    Output structure for the getStylesheet operation.
    
    :var outputs: List of StylesheetOutput objects, one for each input object.
    :var serviceData: This service data contains any partial errors which may have been encountered during processing. 
    The partial error client Ids match the client Id for the input which failed.
    """
    outputs: List[StylesheetOutput] = ()
    serviceData: ServiceData = None


@dataclass
class StylesheetInputData(TcBaseObj):
    """
    The StylesheetInputData structure contains all the data required in order to a server to process the input object
    and return the result to the client.
    
    :var clientId: This is a unique identifier in order to look up Partial Errors and find corresponding output
    structures.
    :var boName: The business object type name.  It is used to determine the stylesheet to load.  If the boReference
    field is input, then this field is ignored.
    :var boReference: The object for which to load a stylesheet.  If this field is input then the boName field is
    ignored.
    :var stylesheetType: The stylesheet type to search for. The value may be "Summary", "Form", "Create", "SaveAs", or
    "Properties".
    """
    clientId: str = ''
    boName: str = ''
    boReference: BusinessObject = None
    stylesheetType: str = ''


@dataclass
class StylesheetOutput(TcBaseObj):
    """
    The StylesheetOutput structure contains all the data required for a client to present the stylesheet to the user
    for the input object.
    
    :var clientId: This is the client Id which was sent in on input.
    :var xrtStylesheet: The XRT style sheet, if XRT is to be returned.
    :var htmlStylesheet: The HTML stylesheet, if HTML is to be returned.
    :var childrenMap: The map of property names on the input type to children(string/vector<BusinessObject>) which are
    to be displayed in UI.
    :var thumbnailFileTicketMap: A map of child BusinessObject to its thumbnail file ticket (business object/string).
    """
    clientId: str = ''
    xrtStylesheet: str = ''
    htmlStylesheet: str = ''
    childrenMap: StylesheetChildrenMap = None
    thumbnailFileTicketMap: FileTicketMap = None


@dataclass
class DatasetInfo(TcBaseObj):
    """
    This structure contains the ZIPFILE file tickets and last mod date for the dataset.
    
    :var lastModDate: This field contains the dataset's last modified date.
    :var fileTickets: This is the vector of ZIPFILE file tickets for the dataset.
    """
    lastModDate: datetime = None
    fileTickets: List[str] = ()


"""
This is a map of dataset name to DatasetInfo object.
"""
DatasetInfoMap = Dict[str, DatasetInfo]


"""
Map from business object to thumbnail file ticket.
"""
FileTicketMap = Dict[BusinessObject, str]


"""
Map of property names to child objects.
"""
StylesheetChildrenMap = Dict[str, List[BusinessObject]]



from __future__ import annotations

from tcsoa.gen.StructureManagement._2012_02.StructureVerification import EquivalentLines
from tcsoa.gen.Internal.StructureManagement._2018_11.MassUpdate import SaveImpactedAssembliesIn, HasActiveMarkupAssociatedOut
from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.gen.Internal.StructureManagement._2018_11.StructureVerification import MountAttachComparisonsResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.StructureManagement._2012_10.StructureVerification import StringToPartialMatchCriteria
from tcsoa.base import TcService


class MassUpdateService(TcService):

    @classmethod
    def hasActiveMarkupAssociated(cls, changeObject: ItemRevision) -> HasActiveMarkupAssociatedOut:
        """
        This operation checks if there are active Mass Update markups associated with the impacted items attached to
        the change object.
        
        Use cases:
        User tries to remove problem object from the change object from Mass Update, this will remove all the
        associated markups, if exists. Before removing, the existence of associated markups will be checked and
        confirmation message will be shown to the user.
        """
        return cls.execute_soa_method(
            method_name='hasActiveMarkupAssociated',
            library='Internal-StructureManagement',
            service_date='2018_11',
            service_name='MassUpdate',
            params={'changeObject': changeObject},
            response_cls=HasActiveMarkupAssociatedOut,
        )

    @classmethod
    def saveImpactedAssemblies(cls, changeObject: ItemRevision, impactedObjectsInfo: List[SaveImpactedAssembliesIn]) -> ServiceData:
        """
        This operation saves the proposed changes in the form of markup and markup change on parents BOM View Revision,
        based on the modified properties on Fnd0MUImpactedParents will add, update or delete markup changes and add
        parent BOM View revision.
        
        Use cases:
        Once user attached Problem Object to ChangeItemRevision, impacted assemblies listed on Mass Update.
        User can select Action (Add Part,Replace part, Remove Part and Substitutes &amp; Add Part as substitute) &amp; 
        its corresponding Proposed Solution Object, this operation saves action and proposed object as Fnd0Markup
        objects.
        """
        return cls.execute_soa_method(
            method_name='saveImpactedAssemblies',
            library='Internal-StructureManagement',
            service_date='2018_11',
            service_name='MassUpdate',
            params={'changeObject': changeObject, 'impactedObjectsInfo': impactedObjectsInfo},
            response_cls=ServiceData,
        )


class StructureVerificationService(TcService):

    @classmethod
    def getMountAttachComparisonDetails(cls, equivalentObjects: List[EquivalentLines], partialMatchCriteria: List[StringToPartialMatchCriteria]) -> MountAttachComparisonsResponse:
        """
        This operation returns the details of any differences between physical attachments including their relation
        properties for the supplied source and target BOMLine objects. 
        
        Physical attachments are secondary resources like tool or weld gun objects connected using
        Mfg0MEPhysicalAttachment and Mfg0MEMountToolToRobot relations. The operation takes the source and target
        BOMLine objects and compares their physical connection according to their types.
        
        For the equivalent primary object, the secondary should be equivalent, for example - tools are compared with
        tools and weld guns with weld guns. Also it compares the properties defined on the relation between physical
        attachments of source and target BOMLine objects. 
        
        The supported physical attachment relation types to compare are Mfg0MEPhysicalAttachment and
        Mfg0MEMountToolToRobot. 
        
        The secondary resources with their physical attachment relation from source and target objects are returned by
        this operation in the form of a table that is created by the output structures.
        
        Use cases:
        User creates an Alternative Collaboration Context (CC) structure from a Plant structure and Bill Of Equipment
        (BOE) structure. In the BOE structure, the resources have one or more physical attachments between them
        connected using Mfg0MEPhysicalAttachment and Mfg0MEMountToolToRobot relation. The mount and attach properties
        of the resources are defined on these relations.
        
        User sends such Alternative CC to Line Designer or Process Simulate application and manipulates mount or attach
        properties of these resources by performing actions.
        
        Now, the user wants to perform comparison of resources (attached or mounted as tools or weld guns) including
        properties defined on the relations between them within a particular scope in Teamcenter.
        """
        return cls.execute_soa_method(
            method_name='getMountAttachComparisonDetails',
            library='Internal-StructureManagement',
            service_date='2018_11',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'partialMatchCriteria': partialMatchCriteria},
            response_cls=MountAttachComparisonsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BOMWindow, BusinessObject, RevisionRule
from typing import List
from tcsoa.gen.Internal.Rdv._2009_01.VariantManagement import MetaExprTokens
from tcsoa.gen.Internal.Rdv._2009_05.VariantManagement import SearchResponse, SearchResults, NVEMetaExprTokensResponse
from tcsoa.base import TcService


class VariantManagementService(TcService):

    @classmethod
    def realignNVEMetaExpressionTokens(cls, metaExprs: List[MetaExprTokens]) -> NVEMetaExprTokensResponse:
        """
        This operation realigns the 'MetaExprTokens'. Meta expressions are used to balance the tree structure amongst
        the variant expressions and validate the input combination of Named Variant Expressions (NVEs). It creates the
        NVEMetaExpression object from 'MetaExprTokens' object and realigns it. The operation returns this realigned
        'MetaExprTokens' list along with 'ServiceData' object.
        
        
        Use cases:
        This operation can be used to realign the Named Variant Expression tokens to make sure that they do not equate
        to always false condition. Input parameter takes the list of MetaExprTokens those needs to be realigned.
        """
        return cls.execute_soa_method(
            method_name='realignNVEMetaExpressionTokens',
            library='Internal-Rdv',
            service_date='2009_05',
            service_name='VariantManagement',
            params={'metaExprs': metaExprs},
            response_cls=NVEMetaExprTokensResponse,
        )

    @classmethod
    def executeAdhocSearchWithOverlays(cls, datasetTag: BusinessObject, productRevTag: BusinessObject, bookMarkStringsToMatch: List[str], bomWindow: BOMWindow, targetBOMLine: List[BOMLine], vooFlag: bool, svrTag: BusinessObject) -> SearchResponse:
        """
        This method performs an Adhoc Search and Valid Overlays Only analysis on the search results, using the
        following input parameters:
        - datasetTag
        - productRevTag
        - bookMarkStringsToMatch
        - bomWindow
        - targetBOMLine
        - vooFlag
        - svrTag
        
        
        
        The datasetTag is a DirectModelAssembly dataset which contains the unconfigured structure information,
        generated using the BOMWriter. The unconfigured plmxml file, attached with the dataset, is parsed and processed
        to create a pruned plmxml file, containing the search result. This file can be opened in TcVis, in order to
        view the results. The method also performs Valid Overlays Only filtering, based on the target BOMLine objects
        and the Saved VariantRule object which are used to the effective variant condition is calculated, to perform
        the VOO filtering.
        This method is currently used by the Teamcenter Rich Client Adhoc DC application, to perform Adhoc Search and
        VOO filtering.
        
        Use cases:
        Use case1: Executing Adhoc Search along with Valid Overlays Only filter
        If the user wishes to execute Adhoc Search and filter the search results using valid overlays filter, then this
        operation is invoked. It uses the effective variant condition on the target BOMLine objects along with the
        Saved VariantRule to apply the filter.
        """
        return cls.execute_soa_method(
            method_name='executeAdhocSearchWithOverlays',
            library='Internal-Rdv',
            service_date='2009_05',
            service_name='VariantManagement',
            params={'datasetTag': datasetTag, 'productRevTag': productRevTag, 'bookMarkStringsToMatch': bookMarkStringsToMatch, 'bomWindow': bomWindow, 'targetBOMLine': targetBOMLine, 'vooFlag': vooFlag, 'svrTag': svrTag},
            response_cls=SearchResponse,
        )

    @classmethod
    def executeSearchWithOverlays(cls, bookMarkStringsToMatch: List[str], bomWindow: BOMWindow, revRule: RevisionRule, targetBOMLine: List[BOMLine], vooFlag: bool) -> SearchResults:
        """
        This method categorizes the input bookmark strings into Configured BOMLine objects that match the input
        strings, Unconfigured BOMLine objects that are not configured for the current variant rule or effective date,
        Unconfigurable BOMLine objects for which no revision configured for the current revision rule and Unconfigured
        VOO BOMLine objects that got unconfigured by applying the Valid Overlay Only (VOO) filter, using the different
        input parameters provided.
        The following inputs are required:
        - bookMarkStringsToMatch
        - bomWindow
        - revRule
        - targetBOMLine
        - vooFlag
        
        
        All the input parameters are mandatory.
        
        This method is used in QPL search (NX and JT based) and Appearance based search. As valid overlays filtering is
        in built in the cacheless search, this method is not required in cacheless search engine. The bookmark strings
        expected are different for different search engines.
        
        For NX based QPL search engine, the bookmark string is a BOMLine object string, that consists of an Occurrence
        Note UG ENTITY HANDLE value for BOMLine objects all the way to the top BOMLine for a particular BOMLine object.
        Example:
        ABC12345/001 view,
        DEF12345/001 view ( UG ENTITY HANDLE contains x38 ),
             GHI12345/001 ( UG ENTITY HANDLE contains x162E ),
        The bookmark string that corresponds to the leaf node BOMLine object is the bookmark string //56/5678, and the
        BOMLine object that will be returned is GHI12345/001.
        For a JT based QPL search engine, the bookmark string is a BOMLine object string which is a concatenation of
        the occurrence UIDs on the occurrence tree path, using the string representation.
        ABC12345/001 view,
        DEF12345/001 view ( Occurrence UID = g8_YK3Jyg9jhgD ),
             GHI12345/001 ( Occurrence UID = gMyopk3ag9jhgD ),
        The BOMLine object string that corresponds to the leaf node BOMLine object is //g8_YK3Jyg9jhgD/gMyopk3ag9jhgD,
        and the BOMLine object that will be returned is GHI12345/001.
        
        For an Appearance based search engine, the bookmark string is the concatenation of the UIDs of the appearance.
        
        Use cases:
        Use case 1: Applying valid overlays only filtering on the QPL/Appearance based search results. 
        If the user wishes to filter search results using valid overlays filter, then this operation is invoked. It
        uses the effective variant condition on the target lines provided to filter out the non matching BOMLine
        objects from the search results.
        """
        return cls.execute_soa_method(
            method_name='executeSearchWithOverlays',
            library='Internal-Rdv',
            service_date='2009_05',
            service_name='VariantManagement',
            params={'bookMarkStringsToMatch': bookMarkStringsToMatch, 'bomWindow': bomWindow, 'revRule': revRule, 'targetBOMLine': targetBOMLine, 'vooFlag': vooFlag},
            response_cls=SearchResults,
        )



from tcsoa.gen.Internal_Workflowaw._2020_12.services import WorkflowService as imp0
from tcsoa.base import TcService


class WorkflowService(TcService):
    getWorkflowTaskAssignments = imp0.getWorkflowTaskAssignments
    updateWorkflowTaskAssignments = imp0.updateWorkflowTaskAssignments



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2008_05.Structure import BOMLinePair
from typing import List
from tcsoa.gen.Internal.StructureManagement._2008_05.Restructure import ReplaceInContextParameter
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class RestructureService(TcService):

    @classmethod
    def replaceInContext(cls, input: List[ReplaceInContextParameter]) -> ServiceData:
        """
        Replaces the line with a newly created item copied from the item to be replaced and copy the children from the
        original item, absolute occurrences, GRM links and Incremental Change Elements of the children will be copied.
        
        Note:
        o  The operation only supports items. Objects of General Design Elements are not supported.
        o  The item used to replace the selected line should be copied from the selected item by calling save as
        action, so that the occurrences of the old children will have clone stable id associated to the new children.
        o  The operation will invoke replace action, so restrictions to replace action will also apply to this
        operation:
        - Preference PS_replace_with_substructure will also apply
        - If the selected line has substitutes, the operation will fail.
        - If the selected line or its parent is linked to variant item, the operation will fail.
        - If the selected line is a variant item, the item to replace it should be a matching variant item.
        
        
        
        Use cases:
        User has a three level structure which has absolute occurrence data at leaf nodes. He saves the parent of the
        leaf node and save it as a new item and invokes this operation to replace the existing line while keep the
        children and absolute occurrence data.
        """
        return cls.execute_soa_method(
            method_name='replaceInContext',
            library='Internal-StructureManagement',
            service_date='2008_05',
            service_name='Restructure',
            params={'input': input},
            response_cls=ServiceData,
        )


class StructureService(TcService):

    @classmethod
    def syncAlignedOccurrences(cls, input: List[BOMLinePair]) -> ServiceData:
        """
        When alignment is out of sync, copy specified property values from source of the alignment to the target.
        
        Note:
        - Properties to be synchronized are determined by preference MEAlignedPropertiesList and
        MEAlignedPropertiesToExcludeFromSync.
        - Properties in out of the box preference MEAlignedPropertiesList are supported. Properties not in the list
        should be supported unless they require complex handling.
        - Use property 'bl_occ_effectivity' for occurrence effectivity. Legacy occurrence effectivity is not support.
        If the occurrence effectivity is shared, then EBOM and MBOM will share it, otherwise it will be copied.
        Preference CFMOccEffMode can be used to configure occurrence effectivity mode.
        - Only legacy variant condition is supported by using property 'bl_condition_tag', and condition is shared.
        Modular variant condition is not supported.
        
        
        
        Use cases:
        User finds out of sync alignments by calling operation 'checkAlignment', and calls this operation to
        synchronize the alignments.
        """
        return cls.execute_soa_method(
            method_name='syncAlignedOccurrences',
            library='Internal-StructureManagement',
            service_date='2008_05',
            service_name='Structure',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Cad._2007_09.StructureManagement import CreateOrUpdateVariantCondInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def createOrUpdateVariantConditions2(cls, inputObjects: List[CreateOrUpdateVariantCondInput]) -> ServiceData:
        """
        This operation is to 'create' or 'update' (depending on the Operation) a variantCondition ( which is variant
        expression of type load if) for a BOMLine object.
        
        Use cases:
        This operation will be used when user wants to create a new or update an existing classic variant condition for
        a given BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateVariantConditions2',
            library='Cad',
            service_date='2007_09',
            service_name='StructureManagement',
            params={'inputObjects': inputObjects},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.base import TcService
from tcsoa.gen.Internal.ProjectManagement._2009_10.ScheduleManagement import TranslatorResponseScheduleModifyContainer, TranslatorResponseContainer, TranslationDataContainer


class ScheduleManagementService(TcService):

    @classmethod
    def translateOne(cls, scheduleUid: str) -> TranslatorResponseContainer:
        """
        this is a non functional internal soa that would never be called. It is a tagging method for
        TranslatorResponseContainer structure which is used internally to transfer objects between C++ and Java codes.
        """
        return cls.execute_soa_method(
            method_name='translateOne',
            library='Internal-ProjectManagement',
            service_date='2009_10',
            service_name='ScheduleManagement',
            params={'scheduleUid': scheduleUid},
            response_cls=TranslatorResponseContainer,
        )

    @classmethod
    def translateThree(cls, scheduleUid: str) -> TranslationDataContainer:
        """
        Dummy  internal operation
        """
        return cls.execute_soa_method(
            method_name='translateThree',
            library='Internal-ProjectManagement',
            service_date='2009_10',
            service_name='ScheduleManagement',
            params={'scheduleUid': scheduleUid},
            response_cls=TranslationDataContainer,
        )

    @classmethod
    def translateTwo(cls, scheduleUid: str) -> TranslatorResponseScheduleModifyContainer:
        """
        this is a non functional internal soa that would never be called. It is a tagging method for 
        TranslatorResponseScheduleModifyContainer structure which is used internally to transfer objects between C++
        and Java codes.
        Without the tagging the soa generate command would fail.
        """
        return cls.execute_soa_method(
            method_name='translateTwo',
            library='Internal-ProjectManagement',
            service_date='2009_10',
            service_name='ScheduleManagement',
            params={'scheduleUid': scheduleUid},
            response_cls=TranslatorResponseScheduleModifyContainer,
        )



from __future__ import annotations

from tcsoa.gen.Core._2016_05.DataManagement import PropData, GenerateContextSpecificIDsResponse, OptionsMap, SetPropsAndDetectOverwriteResponse, GenerateContextIDsInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def resetContextID(cls, contextNames: List[str]) -> ServiceData:
        """
        This service enables the client to reset the ID for given context names. When the IDs for a context name are
        reset, ID generation will begain from beginning value.
        
        WARNING: Be advised that if a client resets the ID for a context name, it is possible that repeated IDs will be
        returned from generateContextSpecificIDs service for that context name.
        
        Use cases:
        A client has a context name for which it has generated IDs and now wants to generate the IDs for that context
        name again from the beginning. Client calls this Teamcenter service to reset the ID for this context name. The
        next time the client calls generateContextSpecificIDs for this context block of returned IDs starts from the
        beginning value 0.
        """
        return cls.execute_soa_method(
            method_name='resetContextID',
            library='Core',
            service_date='2016_05',
            service_name='DataManagement',
            params={'contextNames': contextNames},
            response_cls=ServiceData,
        )

    @classmethod
    def setPropertiesAndDetectOverwrite(cls, propData: List[PropData], options: OptionsMap) -> SetPropsAndDetectOverwriteResponse:
        """
        This operation detects the overwrite condition for the business object instances if those are modified by any
        other session concurrently and updates the remaining objects with the new property values provided.
        
        Use this operation if the overwrite detection is required to avoid any unintentional overwriting of the objects
        due to concurrent modification of the object.
        
        The overwrite condition is detected only if the preference TC_overwrite_protection is set to "true", else the
        objects will be overwritten with new property values.
        
        Overwrite condition is detected by comparing database value of a property with the old value of a property
        available with client for the object.
        
        Each client using this SOA operation must remember old values to pass to this operation for detection of
        overwrite condition in addition to new/updated values.
        
        Also see Teamcenter::Soa::Core::2010_09::setProperties and Teamcenter::Soa::Core::2007_01::setProperties
        operations.
        
        Use cases:
        1. If no options are provided, the overwrite condition is detected based on old values and database values
        comparision. Only the objects, for which overwrite condition is not detected, are saved. For other objects,
        error will be returned.
        2. If the option USE_LAST_SAVED_DATE is true, overwrite condition is detected based on last saved date in
        addition to the old and database values comparision. 
        3. When IGNORE_OVERWRITE_DETECTION is  true, no overwrite condition is detected and all objects are overwritten
        with the values provided.
        4. When CHECK_IN_CHECKED_OUT_OBJECTS is provided with value "true", the objects will be checked in after
        setting the properties of the objects. The objects which are detected for overwrite condition are not checked
        in.
        5. When ENABLE_PSE_BULLETIN_BOARD is "true", it is used to enable the generation of PSE bulletin board events.
        These events are processed through Bulletin board callback mechanism.
        6. When ERROR_MODIFYING_NOT_CHECKEDOUT_OBJECTS is "true", the objects which are not checked-out will not be
        saved and a partial error 32015 for that object will be returned.
        """
        return cls.execute_soa_method(
            method_name='setPropertiesAndDetectOverwrite',
            library='Core',
            service_date='2016_05',
            service_name='DataManagement',
            params={'propData': propData, 'options': options},
            response_cls=SetPropsAndDetectOverwriteResponse,
        )

    @classmethod
    def generateContextSpecificIDs(cls, generateContextIDsIn: List[GenerateContextIDsInput]) -> GenerateContextSpecificIDsResponse:
        """
        Generates the range of unique IDs for input context names. The number of IDs generated for each context name
        depends on the input. If for a given context name, the ID has been reset using Teamcenter service
        resetContextID, then this service generates IDs for that context from the beginning.
        ID generation will also reset when the maximum limit is met. This limit is maximum number supported on 64 bit
        machine.
        
        WARNING: IDs generated using this service  are unique within a given context name, but are not guaranteed to be
        unique in all Teamcenter contextx. Caution should be used if requesting ids for item or other Teamcenter
        objects that require unique ids. The caller may choose to validate uniqueness in the use cases. By default
        Teamcenter will not allow an object be saved if it violates defined uniqueness criteria.
        
        Use cases:
        A user has a context name for which he wants to generate IDs. The user provides the context name and the number
        for IDs to be generated to this Teamcenter service. In response the user recives a block of IDs. If the user
        again uses this service to generate additional IDs for the same context name, new IDs are generated and
        returned in the response structure. The IDs generated in two calls of this service for a given context name are
        unique unless the service resetContextID has been called for that context between the two calls to generate IDs.
        """
        return cls.execute_soa_method(
            method_name='generateContextSpecificIDs',
            library='Core',
            service_date='2016_05',
            service_name='DataManagement',
            params={'generateContextIDsIn': generateContextIDsIn},
            response_cls=GenerateContextSpecificIDsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Classification._2009_10.Classification import KeyLOVDefinition2, ExtendedProeprty, AttributeConfiguration
from tcsoa.gen.Classification._2007_01.Classification import ClassDef, AttributeFormat
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class FormatPropertiesNX(TcBaseObj):
    """
    Structure representing format details
    
    :var format: The format definition for this attribute.
    :var unitDisplayName: Unit display name associated with this attribute in this unit system.
    :var unitID: Unit ID associated with this attribute in this unit system.
    :var nxUnitID: NX Unit ID associated with this attribute in this unit system.
    :var defaultValue: Default value of this Class attribute. This can be an empty string indicating no default value.
    :var minimumValue: Minimum value constraint of this Class attribute. This can be an empty string indicating no
    minimum value constraint.
    Note: Only applicable to numerical formats of attributes
    :var maximumValue: Maximum value constraint of this Class attribute. This can be an empty string indicating no
    maximum value constraint.
    Note: Only applicable to numerical formats of attributes
    """
    format: AttributeFormat = None
    unitDisplayName: str = ''
    unitID: str = ''
    nxUnitID: str = ''
    defaultValue: str = ''
    minimumValue: str = ''
    maximumValue: str = ''


@dataclass
class GetClassDefinitionsResponseNX(TcBaseObj):
    """
    Holds the values returned by getClassDefinitionsNX operation.
    
    :var classDefinitionMap: A map (string, ClassDef) of Classification class or view ID and its corresponding
    definition pairs.
    :var classAttributesMap: A map (string, ClassAttributesDefinitionNX) of Classification class or view ID and its
    corresponding class attributes.
    :var keylovDefintion: A map (string ID, KeyLOVDefinition2) containing the used Classification KeyLOVs and its
    definitions.
    :var serviceData: Any failures will be returned in the service data list of partial errors.
    """
    classDefinitionMap: ClassDefinitionsNX = None
    classAttributesMap: ClassAttributesDefinitionMapNX = None
    keylovDefintion: KeylovDefinitionsNX = None
    serviceData: ServiceData = None


@dataclass
class ClassAttributeNX(TcBaseObj):
    """
    Structure representing class attribute details.
    
    :var id: ID for this attribute.
    :var name: Name for this attribute.
    :var dependencyAttribute: The dependency attribute property of this attribute.
    :var dependencyConfiguration: The dependency configuration property of this attribute.
    :var extendedProperties: A list of references to the ExtendedProeprty structure holding the extended metadata
    properties of this attribute.
    :var shortName: Short name defined for this attribute.
    :var description: Description added for this attribute.
    :var metricFormat: The metric format of this attribute.
    :var nonMetricFormat: The nonmetric format of this attribute.
    :var annotation: Annotation information added to this attribute.
    :var attributeConfiguration: Additional configuration for this attribute.
    :var arraySize: Array size or the number of values for this attribute. 
    - If single valued (non-array), then arraySize = 1.
    - If multi valued (array), then arraySize >= 1 corresponding to the size of the array defined in the attribute
    definition.
    
    
    :var options: Attribute property flags represented as a single integer. To access individual property flags, a
    bitwise operation will be required by the client. Valid values are:
    - ATTR_vla = (1 << 0) 
    - ATTR_external_vla = (1 << 1) 
    - ATTR_mandatory = (1 << 2) 
    - ATTR_protected = (1 << 3) 
    - ATTR_unique = (1 << 4) 
    - ATTR_propagated = (1 << 5) 
    - ATTR_localValue = (1 << 6) 
    - ATTR_reference = (1 << 7) 
    - ATTR_auto_computed = (1 << 15) 
    - ATTR_hidden = (1 << 20) 
    - ATTR_localizable = (1 << 22 )
    
    """
    id: int = 0
    name: str = ''
    dependencyAttribute: str = ''
    dependencyConfiguration: str = ''
    extendedProperties: List[ExtendedProeprty] = ()
    shortName: str = ''
    description: str = ''
    metricFormat: FormatPropertiesNX = None
    nonMetricFormat: FormatPropertiesNX = None
    annotation: str = ''
    attributeConfiguration: AttributeConfiguration = None
    arraySize: int = 0
    options: int = 0


@dataclass
class ClassAttributesDefinitionNX(TcBaseObj):
    """
    The structure containing list of Classification class attributes definition and configured KeyLOV (stxt) definition.
    
    :var classAttributes: The list of attributes defined for the classification class.
    :var configuredKeyLOVs: A map (int, KeyLOVDefinition2) of attribute ID and KeyLOV definition pairs, based on
    dependency configuration of an attribute.
    """
    classAttributes: List[ClassAttributeNX] = ()
    configuredKeyLOVs: ConfiguredKeyLOVDefinitionNX = None


"""
The list of KeyLOV (Stxt) id and corresponding KeyLOV (Stxt) definition.
"""
KeylovDefinitionsNX = Dict[str, KeyLOVDefinition2]


"""
The map of Classification Class or View ID and corresponding attributes definition.
"""
ClassAttributesDefinitionMapNX = Dict[str, ClassAttributesDefinitionNX]


"""
The map of Classification Class or View ID and corresponding definition.
"""
ClassDefinitionsNX = Dict[str, ClassDef]


"""
The list of attribute ID and corresponding configured KeyLOV (stxt) definition based on dependency configuration set on class attribute.
"""
ConfiguredKeyLOVDefinitionNX = Dict[int, KeyLOVDefinition2]



from __future__ import annotations

from tcsoa.gen.Query._2007_06.SavedQuery import RetrieveSearchCriteriaResponse, SavedQueryInput, ExecuteSavedQueriesResponse, SaveSearchCriteriaInfo
from tcsoa.gen.Query._2007_06.Finder import FindWorkspaceObjectsResponse, WSOFindSet
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def retrieveSearchCriteria(cls, searchNames: List[str]) -> RetrieveSearchCriteriaResponse:
        """
        Retrieve the corresponding search criteria for the given saved search names.
        """
        return cls.execute_soa_method(
            method_name='retrieveSearchCriteria',
            library='Query',
            service_date='2007_06',
            service_name='SavedQuery',
            params={'searchNames': searchNames},
            response_cls=RetrieveSearchCriteriaResponse,
        )

    @classmethod
    def saveSearchCriteria(cls, searchCriteria: List[SaveSearchCriteriaInfo]) -> ServiceData:
        """
        Save a set of search criteria.  Each search criteria pertains to 
         a saved search, a collection of which is known as "My Saved Searches".
        """
        return cls.execute_soa_method(
            method_name='saveSearchCriteria',
            library='Query',
            service_date='2007_06',
            service_name='SavedQuery',
            params={'searchCriteria': searchCriteria},
            response_cls=ServiceData,
        )

    @classmethod
    def executeSavedQueries(cls, input: List[SavedQueryInput]) -> ExecuteSavedQueriesResponse:
        """
        Executes a set of saved queries of following type:
        
        - QRY_RUN_BY_TC ( 0 )
        - QRY_RUN_BY_USER_EXIT ( 8 )
        - QRY_RUN_BY_KEYWORD_SEARCH ( 24 )
        - QRY_RUN_BY_EINT_EXIT ( 32 )
        - QRY_RUN_BY_TC_PLUS_PROCESS ( 56 )
        
        
          
        The saved queries can be executed to yield results in 2 modes:
        
        - Flat mode: In this traditional execution mode, only the first-level  objects (corresponding to         the
        queried class) satisfying the query  are returned
        - Hierarchical/Indented mode: This mode is only applicable for saved queries that allow        
        Hierarchical/Indented results. In this execution mode, the first-level objects as well as any         sub-level
        objects satisfying the query criteria are returned. The hierarchy level information is         also returned so
        that result subsets can be  re-constructed using the resulting objects. 
        
        
                 This service will return the matched object UIDs.   After fetching UIDs, client needs to call        
        DataManagementService.loadObjects operation to load objects by pages.  See the         QueryInput and
        QueryResults data structures for usage details.
        
        Use cases:
        Execute a set of saved queries of following types: QRY_RUN_BY_TC,  QRY_RUN_BY_KEYWORD_SEARCH and
        QRY_RUN_BY_TC_PLUS_PROCESS etc.
        """
        return cls.execute_soa_method(
            method_name='executeSavedQueries',
            library='Query',
            service_date='2007_06',
            service_name='SavedQuery',
            params={'input': input},
            response_cls=ExecuteSavedQueriesResponse,
        )


class FinderService(TcService):

    @classmethod
    def findWorkspaceObjects(cls, findList: List[WSOFindSet]) -> FindWorkspaceObjectsResponse:
        """
        Query the database for WorkspaceObjects. A collection of WSOFindSets are used to do the queries. For each
        WSOFindSet , a FindWorkspaceObjectsOutput will be generated if any WorkspaceObjects are found that meet all the
        criteria.  Each FindWorkspaceObjectsOutput will contain the tags of the WorkspaceObjects that meet all the
        criteria and the index of the WSOFindSet in the findList that generated the output.  If an error is
        encountered, then no FindWorkspaceObjectsOutput will be generated for that WSOFindSet (no partial data
        returned) and the index for the WSOFindSet in the findList will be the client ID in the partial error.  If no
        WSOFindSet generates any output, a null outputList is returned.
        
        Use cases:
        Find workspace object using basic attributes such as object type, object name, owner, group, created before,
        created after etc.
        """
        return cls.execute_soa_method(
            method_name='findWorkspaceObjects',
            library='Query',
            service_date='2007_06',
            service_name='Finder',
            params={'findList': findList},
            response_cls=FindWorkspaceObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awb0Element
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class OccurrenceCursor(TcBaseObj):
    """
    OccurrenceCursor is a cursor that is returned for use in subsequent call to retrieve the next page of configured
    occurrences.
    
    :var startReached: If true, the first page of the results has been reached.
    :var endReached: If true, the last page of the results has been reached.
    :var startIndex: Indicates the Cursor start position for the result occurrences returned so far.
    :var endIndex: Indicates the Cursor end position for the result occurrences returned so far.
    :var pageSize: Indicates the maximum number of occurrences that can be returned in one service call.
    :var startOccUid: Indicates the cursor start object for the result occurrences returned so far. This is used for
    flat mode only.
    :var endOccUid: Indicates the cursor end object for the result occurrences returned so far. This is used for flat
    mode only.
    :var cursorData: The cursor data.
    """
    startReached: bool = False
    endReached: bool = False
    startIndex: int = 0
    endIndex: int = 0
    pageSize: int = 0
    startOccUid: str = ''
    endOccUid: str = ''
    cursorData: List[str] = ()


@dataclass
class OccurrenceInfo6(TcBaseObj):
    """
    OccurrenceInfo6 is the information about occurrence that is part of the configuration result and passing the filter
    critiera.
    
    :var occurrence: Instance of the Awb0Element.
    :var occurrenceId: Session Recoverable UID of the configured occurrence.
    :var stableId: Stable UID of Awb0Element. This  strictly used for maintaining expansion state of nodes in tree view.
    :var displayName: Display name of the configured occurrence.
    :var numberOfChildren: Number of children of the configured occurrence.
    :var underlyingObjectType: Internal name of the underlying object that occurrence presents.
    """
    occurrence: Awb0Element = None
    occurrenceId: str = ''
    stableId: str = ''
    displayName: str = ''
    numberOfChildren: int = 0
    underlyingObjectType: str = ''


@dataclass
class PackedOccurrenceCSIDsResponse(TcBaseObj):
    """
    Represents the response for getPackedOccurrenceCSIDs operation. PackedOccurrenceCSIDsResponse contains the clone
    stable ids of packed occurrences.
    
    :var csids: The clone stable ids for the input packed occurrences.
    :var serviceData: The Service Data.
    """
    csids: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class UserContextInfo(TcBaseObj):
    """
    UserContextInfo contains user's working state information like active page, last saved session time for the opened
    product.
    
    :var autoSavedSessiontime: The auto saved session time for the opened product and the logged in user.
    :var sublocationAttributes: A map (string,list of strings) of client state attribute name/value pairs. The
    following keys are supported: 'awb0ActiveSublocation'. &lsquo;awb0ActiveSublocation&rsquo; refers to name of tabs
    like &lsquo;Overview&rsquo;, &lsquo;Viewer&rsquo;.
    """
    autoSavedSessiontime: datetime = None
    sublocationAttributes: SublocationAttributes2 = None


@dataclass
class FocusOccurrenceInput(TcBaseObj):
    """
    Contains input information required to set the focusChildOccurrence.
    
    :var element: The focus child Business Object for which sibling occurrences are desired. Valid Business Objects
    types are Awb0Element, Cpd0DesignElement, Ptn0Partition.
    :var cloneStableIdChain: Clone stable id of child occurrence in top-down fashion. The chain is separated by "/".
    This field is considered only when focusChildOccurrence is empty.
    """
    element: BusinessObject = None
    cloneStableIdChain: str = ''


"""
Map (string, list of string) of client state attribute name and value pairs. At this time only one (key) attribute name is supported: "activeSublocation". Supported values are: (tab names like) "overview", "viewer" . Keys and values are case in-sensitive.
"""
SublocationAttributes2 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportToApplicationInputData(TcBaseObj):
    """
    The ExportToApplicationInputData structure represents all of the data necessary to export selected objects to
    Word/Excel.
    
    :var objectsToExport: The list of Teamcenter business objects to export.
    :var attributesToExport: The list of attributes to export.
    :var applicationFormat: The application format such as "'MSWordXML'", "'MSExcel'" and "'MSExcelLive'".
    
    Supported application format for this operation
    
    - MSWordXML     This format is used for exporting WorkspaceObject to static MSWord application.
    - MSExcel    This format is used for exporting Teamcenter objects to static MSExcel  application.
    - MSExcelLive    This format is used for exporting Teamcenter objects to Live MSExcel  application.
    
    
    :var templateId: The name of the Word/Excel template.
    """
    objectsToExport: List[BusinessObject] = ()
    attributesToExport: List[str] = ()
    applicationFormat: str = ''
    templateId: str = ''


@dataclass
class ExportToApplicationResponse(TcBaseObj):
    """
    ExportToApplicationResponse structure represents the output of export to application operation.
    It has information about file ticket for the exported file generated at the server.
    
    :var transientFileReadTickets: The transient file read tickets for the exported file.
    :var serviceData: The Service Data
    """
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ImportFromApplicationInputData(TcBaseObj):
    """
    This holds the data necessary to import an MSWord document into Teamcenter.
    
    :var transientFileWriteTicket: The file ticket of the. docx file to be imported into Teamcenter.
    :var applicationFormat: The supported application format is MSWordXML
    
    Supported application format for this operation
    
    - MSWordXML     This format is used for exporting WorkspaceObjects business objects to static MSWord application.
    
    
    :var createSpecElementType: The subtype of SpecElement to be created.
    """
    transientFileWriteTicket: str = ''
    applicationFormat: str = ''
    createSpecElementType: str = ''


@dataclass
class ImportFromApplicationResponse(TcBaseObj):
    """
    ImportFromApplicationResponse structure represents the output of import from application operation. It contains the
    UID of the BOMWindow after the document is imported to Teamcenter.
    
    :var resultObjects: The resultant objects which contains the UID of the BOMWindow created after the document is
    imported.  In case of importing templates, it contains the tag of the template.
    :var serviceData: The Service Data
    """
    resultObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Group, Role
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class RoleGroupStructure(TcBaseObj):
    """
    A structure of Role objects and the associated Group object.
    
    :var roles: A list of Role objects to be removed.
    :var grp: The Group object from which Role objects are to be removed.
    """
    roles: List[Role] = ()
    grp: Group = None


@dataclass
class RoleStructure(TcBaseObj):
    """
    Properties for the new Role to be created and added to the Group.
    
    :var roleName: Name of the Role to be created.
    :var roleDescription: The Role description.
    """
    roleName: str = ''
    roleDescription: str = ''


@dataclass
class AddRolesToGroupStructure(TcBaseObj):
    """
    Roles to be added to a Group object.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var rolesToAdd: A list of Role objects to be added to the Group.
    :var rolesToCreateAndAdd: A list of Role properties. A new Role will be created for each RoleStructure in the list
    and added to the Group.
    :var grp: Group for the roles to be added to.
    """
    clientId: str = ''
    rolesToAdd: List[Role] = ()
    rolesToCreateAndAdd: List[RoleStructure] = ()
    grp: Group = None



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List


@dataclass
class ThumbnailFileTicketsResponse2(TcBaseObj):
    """
    Return type consisiting of list of thumbnail file tickets of the input list of business objects.
    
    :var thumbnailFileTicketsMap: A map of business object and thumbnail FMS tickets (business object/vector(string)).A
    business object may have multiple thumbnails.
    :var serviceData: The list of partial errors.
    """
    thumbnailFileTicketsMap: ThumbnailFileTicketsMap = None
    serviceData: ServiceData = None


"""
A map of  the business object and the thumbnail file tickets.
"""
ThumbnailFileTicketsMap = Dict[BusinessObject, List[str]]
